# Generated by Django 3.2.6 on 2021-08-07 05:23

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='SOAPRequestLogger',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_sent', models.DateTimeField(auto_now_add=True)),
                ('method', models.CharField(max_length=10)),
                ('url', models.URLField(max_length=255)),
                ('headers', models.TextField(blank=True, null=True)),
                ('body', models.TextField(blank=True, null=True)),
                ('request_time_ms', models.IntegerField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='SOAPResponseLogger',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_received', models.DateTimeField(blank=True, null=True)),
                ('status', models.IntegerField(blank=True, null=True)),
                ('headers', models.TextField(blank=True, null=True)),
                ('body', models.TextField(blank=True, null=True)),
                ('request_id', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='django_soap.soaprequestlogger')),
            ],
        ),
    ]
